from tkinter import Tk, Frame, Label, Entry, Button, messagebox
from main import *
from src.staffUser import StaffUser
from src.adminUser import AdminUser
from src.managerUser import ManagerUser
from main import loginAttempt
from tkinter import Listbox
from src.dbfunc import fetch_movies

import sys
sys.path.append("src")  # Ensure Python finds 'src' modules
from main import *


# Initialize main window
window = Tk()
window.title("Login")
window.geometry('340x440')

# Color for light and dark mode
light_mode = {
    "bg": "#f0f0f0",
    "fg": "#000000",
    "button_bg": "#e0e0e0",
    "button_fg": "#000000",
    "entry_bg": "#ffffff",
    "entry_fg": "#000000",
}
dark_mode = {
    "bg": "#333333",
    "fg": "#ffffff",
    "button_bg": "#555555",
    "button_fg": "#ffffff",
    "entry_bg": "#444444",
    "entry_fg": "#ffffff",
}

# Initialize with dark mode
current_theme = dark_mode

# Toggle between light and dark mode
def toggle_theme():
    global current_theme
    if current_theme == dark_mode:
        current_theme = light_mode
    else:
        current_theme = dark_mode
    apply_theme()

# Apply the selected theme to all widgets
def apply_theme():
    window.configure(bg=current_theme["bg"])
    form_frame.configure(bg=current_theme["bg"])
    login_label.configure(bg=current_theme["bg"], fg=current_theme["fg"])
    username_label.configure(bg=current_theme["bg"], fg=current_theme["fg"])
    password_label.configure(bg=current_theme["bg"], fg=current_theme["fg"])
    username_entry.configure(bg=current_theme["entry_bg"], fg=current_theme["entry_fg"])
    password_entry.configure(bg=current_theme["entry_bg"], fg=current_theme["entry_fg"])
    login_button.configure(bg=current_theme["button_bg"], fg=current_theme["button_fg"])
    theme_toggle_button.configure(bg=current_theme["button_bg"], fg=current_theme["button_fg"])

# Handle login action

# Function - open dashboards based on user role
def open_dashboard(user):
    """Opens different GUI windows based on user role"""
    
    if user is None:
        messagebox.showerror("Error", "Invalid login attempt.")
        return

    dashboard = Tk()
    dashboard.title("Dashboard")
    dashboard.geometry("500x400")

    # Display Welcome Message Using `user.username`
    welcome_label = Label(dashboard, text=f"Welcome, {user.userName}!", font=("Arial", 16))
    welcome_label.pack(pady=20)

    if isinstance(user, StaffUser):
        Label(dashboard, text="Staff Dashboard").pack()
        Button(dashboard, text="View Listings").pack()
        Button(dashboard, text="Create Booking").pack()

    elif isinstance(user, AdminUser):
        Label(dashboard, text="Admin Dashboard").pack()
        Button(dashboard, text="Manage Listings").pack()
        Button(dashboard, text="View Reports").pack()

    elif isinstance(user, ManagerUser):
        Label(dashboard, text="Manager Dashboard").pack()
        Button(dashboard, text="Expand Business").pack()
        Button(dashboard, text="Generate Revenue Reports").pack()

    Button(dashboard, text="Logout", command=dashboard.quit).pack(pady=20)

    dashboard.mainloop()

def gui_login():
    username = username_entry.get()
    password = password_entry.get()
    
    if not username or not password:
        messagebox.showwarning("Warning", "Please fill in both fields.")
        return

    # Call loginAttempt() from main.py
    user = loginAttempt(username, password)  
    #logging.debug(user)

    if user:
        messagebox.showinfo("Login", f"Login successful! Welcome {user.userName}.")
        window.destroy()  # Close login window
        open_dashboard(user)  # Open appropriate dashboard based on user role
    else:
        messagebox.showerror("Error", "Incorrect username or password!")

# Make rows and columns in the main window expand to center the form frame
window.grid_rowconfigure(0, weight=1)
window.grid_rowconfigure(2, weight=1)
window.grid_columnconfigure(0, weight=1)
window.grid_columnconfigure(2, weight=1)

# Login form in the center of the window
form_frame = Frame(window, bg=current_theme["bg"])
form_frame.grid(row=1, column=1, padx=20, pady=20)

# Widgets for login
login_label = Label(form_frame, text="Login", font=("Arial", 18), fg=current_theme["fg"], bg=current_theme["bg"])
username_label = Label(form_frame, text="Username", fg=current_theme["fg"], bg=current_theme["bg"])
username_entry = Entry(form_frame, width=25, bg=current_theme["entry_bg"], fg=current_theme["entry_fg"])
password_label = Label(form_frame, text="Password", fg=current_theme["fg"], bg=current_theme["bg"])
password_entry = Entry(form_frame, show="*", width=25, bg=current_theme["entry_bg"], fg=current_theme["entry_fg"])
login_button = Button(form_frame, text="Login", command=gui_login, bg=current_theme["button_bg"], fg=current_theme["button_fg"], width=20)

# Button for Light/Dark Mode
theme_toggle_button = Button(form_frame, text="Toggle Light/Dark Mode", command=toggle_theme, 
                             bg=current_theme["button_bg"], fg=current_theme["button_fg"])

# Place widgets on the screen
login_label.grid(row=0, column=0, columnspan=2, pady=(10, 10))
username_label.grid(row=1, column=0, padx=5, pady=5, sticky="e")
username_entry.grid(row=1, column=1, padx=5, pady=5, sticky="w")
password_label.grid(row=2, column=0, padx=5, pady=5, sticky="e")
password_entry.grid(row=2, column=1, padx=5, pady=5, sticky="w")
login_button.grid(row=3, column=0, columnspan=2, pady=(10, 10))
theme_toggle_button.grid(row=4, column=0, columnspan=2, pady=(10, 10))

# Enable to use theme
apply_theme()



# app will only launch if run directly from this file.
if __name__ == "__main__":
    window.mainloop()
    
def fetch_movies():
    conn = get_connection()
    if conn:
        cursor = conn.cursor(dictionary=True)
        cursor.execute("SELECT title, genre, age_rating FROM Films")
        movies = cursor.fetchall()
        conn.close()
        return movies
    return []

# Create a listbox widget
movie_listbox = Listbox(dashboard)
movie_listbox.pack()

# Fetch movies from database and display
movies = fetch_movies()
for movie in movies:
    movie_listbox.insert("end", f"{movie['title']} ({movie['genre']}, Rated: {movie['age_rating']})")