import logging
logging.basicConfig(level=logging.DEBUG,
                    format="[{filename}:{lineno}] - {asctime} - {levelname} - {message}",
                    style="{", datefmt="%Y-%m-%d %H:%M")
from src import *
from src.dbfunc import get_connection
import json
import pandas as pd
import datetime as dt
from src.staffUser import StaffUser
from src.adminUser import AdminUser
from src.managerUser import ManagerUser

"""_summary_

- logic should be separate from the GUI
- this file handles interfacing with the database and classes.
- most functionality should also be here
- using main() as a test function during development, and after can be used as a CLI version of the program.

'ideally there should be a sort of intermediary between the GUI and database, for example last year most groups queried the database directly from tkinter, although that's not ideal.' - lecturer idk there name
'you should be able to make python classes directly from your class diagram' - same lecturer

"""


def loginAttempt(username: str, password: str):
    """Check if the user exists in the database and return the appropriate user object
    
    Arguments:
        username (str) -- username for login.
        password (str) -- password for login, plaintext?
    Returns:
        User object of type: StaffUser or AdminUser or ManagerUser
        or None if failed login.
    """
    # dbug shortcut
    if username=="debug" and password=="debug":
        logging.warning("Entered as debug user")
        return ManagerUser(15, "manager", "password", [1], "Manager")
    
    # using dbfuncs to do this:   cursor.execute("SELECT username, password_hash, role FROM users WHERE username = %s AND password_hash = %s", (username, password)) 
    user_data = select_from_table_where("tblUsers", f"username = '{username}' AND password_hash = '{password}'", "user_id, username, password_hash, role, cinemas")
    if user_data == None:
        return None
    user_data = user_data[0]
    logging.debug(user_data)
    
    
    if user_data:
        logging.info(f"User '{user_data['username']}' logged in successfully.")

        # Return an instance of the correct user class
        if user_data['role'] == 'Staff':
            return StaffUser(id=user_data['user_id'], name=user_data['username'], passw=user_data['password_hash'], role=user_data['role'], cinemas=user_data['cinemas'])
        elif user_data['role'] == 'Admin':
            logging.debug(f"created AdminUser object. {user_data['role']}")
            return AdminUser(id=user_data['user_id'], name=user_data['username'], passw=user_data['password_hash'], role=user_data['role'], cinemas=user_data['cinemas'])
        elif user_data['role'] == 'Manager':
            return ManagerUser(id=user_data['user_id'], name=user_data['username'], passw=user_data['password_hash'], role=user_data['role'], cinemas=user_data['cinemas'])
        else:
            logging.error("Unknown role in database.")
            return None
    else:
        logging.warning("Invalid username or password.")
        return None  # Return None if no user found

def veiwBookings_cli(user):
    data = user.veiwBookings()
    logging.debug(f"\n# {data}")
    print("\n\n")
    for i in data:
        print(i)
    
def createBooking_cli(user):
    # input data and validation loop
    while True:
        print("Please input booking info.")
        
        showing_id = int(input("\nShowing ID:"))
        seats = input("\nSeats in format '1' or '1 2 3' (no dupes):")
        if showing_id == "exit" or seats == "exit":
            return
        elif showing_id != None and seats != None:
            status = user.createBooking(showing_id, seats)
            if status == True:
                print("Booking created.")
                createTicket_cli(user, booking_id)
                break
            else:
                print(status)
        else:
            print("Missing input please try again.")

def removeBooking_cli(user):
    booking_id = int(input("\nInput Id of booking to remove: "))
    user.removeBooking(booking_id)

def manageBookings_cli(user):
    veiwBookings_cli(user)
    selected_booking = 0
    # input val loop for booking_id
    while True:
        selected_booking = int(input("\nSelect booking_id to be managed: "))
        logging.debug(f"selected_booking={selected_booking}")
        if selected_booking == None or selected_booking == 0:
            print("Invalid input try again.")
        else:
            selected_booking = select_from_table_where(table="tblBookings", selection="*", condition=f"booking_id={selected_booking}")
            logging.debug(f"selected_booking={selected_booking}")
            if selected_booking is not None:
                break
            else:
                print("db error, try again.") # lol
    # input for new vals
    print("Input new values for booking, or leave blank for no change.")
    user_id = int(input("user_id: "))
    showing_id = int(input("showing_id: "))
    seat_numbers= input("seat_numbers: ")
    total_price= float(input("total_price: "))
    booking_date= input("booking_date: ")
    
    status = user.updateBooking(booking_id=selected_booking, user_id=user_id, showing_id=showing_id, seat_numbers=seat_numbers, total_price=total_price, booking_date=booking_date)
    if status:
        print("Booking updated")
    else:
        print("Failed to update booking.")
    
def createTicket_cli(user, booking_id=None):
    while booking_id == None:
        booking_id = intput("Input booking_id to create a ticket")
        if booking_id.isdigit():
            booking_id = int(booking_id)
            break
        else:
            print("Invalid booking_id.")
    pass
    # create ticket...

def staffUser_options_cli(user:StaffUser) -> None:
    logging.debug("loaded into staffUser_options_cli.")
    while True:
        selection = input("\nMenu options:\n"
                          "  1. View bookings\n"
                          "  2. Create booking \n"
                          "  3. Remove booking\n"
                          "  4. Manage existing booking\n"
                          
                          "  5. Create ticket\n"
                          
                          "  6. Veiw showings\n"
                          
                          "  7. Logout\n"
                          "  please return a number to select: ")
        
        # switch case:
        if selection == "1":
            logging.debug("selection 1, loading View bookings")
            veiwBookings_cli(user)
        elif selection == "2":
            logging.debug("selection 2, loading Create booking")
            createBooking_cli(user)
        elif selection == "3":
            logging.debug("selection 3, Remove booking")
            removeBooking_cli(user)
        elif selection == "4":
            logging.debug("selection 4, Manage existing booking")
            manageBookings_cli(user)
        elif selection == "5":
            logging.debug("selection 5, Create ticket")
            user.createTicket()
        elif selection == "6":
            logging.debug("selection 6, Veiw Showings")
            user.viewShowings()
        elif selection == "7" or selection.lower() == "logout":
            logging.debug("selection 7, Logging out.")
            break
        else:
            print("Invalid input, please input an integer value corresponding with an option.")
    user.logout()
    exit()

def adminUser_options_cli(user:AdminUser) -> None:
    logging.debug("loaded into adminUser_options_cli.")
    while True:
        selection = input("\nMenu options:\n"
                          "  1. View bookings\n"
                          "  2. Create booking \n"
                          "  3. Remove booking\n"
                          "  4. Manage existing booking\n"
                          
                          "  5. Create ticket\n"
                          
                          "  6. Veiw showings\n"
                          "  7. Add showing\n"
                          "  8. Remove showing\n"
                          "  9. Manage existing showing\n"
                          
                          "  10. Veiw films\n"
                          "  11. Add film\n"
                          "  12. Remove film\n"
                          "  13. Manage existing films\n"
                          
                          "  14. Logout\n"
                          "  please return a number to select: ")
        
        # switch case:
        if selection == "1":
            logging.debug("selection 1, loading View bookings")
            user.veiwBookings()
        elif selection == "2":
            logging.debug("selection 2, loading Create booking")
            user.createBooking()
        elif selection == "3":
            logging.debug("selection 3, Remove booking")
            user.removeBooking()
        elif selection == "4":
            logging.debug("selection 4, Manage existing booking")
            user.veiwBookings()
            user.updateBooking()
        elif selection == "5":
            logging.debug("selection 5, Create ticket")
            user.createTicket()
        elif selection == "6":
            logging.debug("selection 6, Veiw Showings")
            user.viewShowings()
        elif selection == "7":
            logging.debug("selection 7, Add showing")
            user.createShowing()
        elif selection == "8":
            logging.debug("selection 8, Remove showing")
            user.removeShowing()
        elif selection == "9":
            logging.debug("selection 9, Manage existing showing")
            user.veiwShowings()
            user.updateShowing()
        elif selection == "10":
            logging.debug("selection 10, Veiw films")
            user.veiwFilms()
        elif selection == "11":
            logging.debug("selection 11, Add film")
            user.createFilm()
        elif selection == "12":
            logging.debug("selection 12, Remove film")
            user.removeFilm()
        elif selection == "13":
            logging.debug("selection 13, Manage existing film")
            user.veiwFilms()
            user.updateFilm()
        elif selection == "14" or selection.lower() == "logout":
            logging.debug("selection 14, Logging out.")
            break
        else:
            print("Invalid input, please input an integer value corresponding with an option.")
    user.logout()
    exit()
    
def managerUser_options_cli(user:ManagerUser) -> None:
    logging.debug("loaded into managerUser_options_cli.")
    while True:
        selection = input("\nMenu options:\n"
                          "  1. View bookings\n"
                          "  2. Create booking \n"
                          "  3. Remove booking\n"
                          "  4. Manage existing booking\n"
                          
                          "  5. Create ticket\n"
                          
                          "  6. Veiw showings\n"
                          "  7. Add showing\n"
                          "  8. Remove showing\n"
                          "  9. Manage existing showing\n"
                          
                          "  10. Veiw films\n"
                          "  11. Add film\n"
                          "  12. Remove film\n"
                          "  13. Manage existing films\n"
                          
                          "  14. Veiw users\n"
                          "  15. Add user\n"
                          "  16. Remove user\n"
                          "  17. Manage existing user\n"
                          
                          "  18. Veiw cinemas\n"
                          "  19. Add cinema\n"
                          "  20. Remove cinema\n"
                          "  21. Manage existing cinema\n"
                          
                          "  22. Logout\n"
                          "  please return a number to select: ")
        
        # uhhhh this is messy af, would love to make this smaller at some other time. XD
        if selection == "1":
            logging.debug("selection 1, loading View bookings")
            data = pd.DataFrame(user.veiwBookings()) # data format is pd.dataframe(list(dict()))
            print(data)
        elif selection == "2":
            logging.debug("selection 2, loading Create booking")
            user.createBooking()
        elif selection == "3":
            logging.debug("selection 3, Remove booking")
            user.removeBooking()
        elif selection == "4":
            logging.debug("selection 4, Manage existing booking")
            user.veiwBookings()
            user.updateBooking()
        elif selection == "5":
            logging.debug("selection 5, Create ticket")
            user.createTicket()
        elif selection == "6":
            logging.debug("selection 6, Veiw Showings")
            data = pd.DataFrame(user.veiwShowings())
            print(data)
        elif selection == "7":
            logging.debug("selection 7, Add showing")
            user.createShowing()
        elif selection == "8":
            logging.debug("selection 8, Remove showing")
            user.removeShowing()
        elif selection == "9":
            logging.debug("selection 9, Manage existing showing")
            user.veiwShowings()
            user.updateShowing()
        elif selection == "10":
            logging.debug("selection 10, Veiw films")
            data = pd.DataFrame(user.veiwFilms())
            print(data)
        elif selection == "11":
            logging.debug("selection 11, Add film")
            user.createFilm()
        elif selection == "12":
            logging.debug("selection 12, Remove film")
            user.removeFilm()
        elif selection == "13":
            logging.debug("selection 13, Manage existing film")
            user.veiwFilms()
            user.updateFilm()
        elif selection == "14":
            logging.debug("selection 14, Veiw users")
            data = pd.DataFrame(user.veiwUsers())
            print(data)
        elif selection == "15":
            logging.debug("selection 15, Add user")
            user.createUser()
        elif selection == "16":
            logging.debug("selection 16, Remove user")
            user.removeUser()
        elif selection == "17":
            logging.debug("selection 17, Manage existing user")
            user.veiwUsers()
            user.updateUser()
        elif selection == "18":
            logging.debug("selection 18, Veiw Cinemas")
            data = pd.DataFrame(user.veiwsCinemas())
            print(data)
        elif selection == "19":
            logging.debug("selection 19, Add Cinema")
            user.createCinema()
        elif selection == "20":
            logging.debug("selection 20, Remove Cinema")
            user.removeCinema()
        elif selection == "21":
            logging.debug("selection 21, Manage existing Cinema")
            user.veiwCinemas()
            user.updateCinema()
        elif selection == "22" or selection.lower() == "logout":
            logging.debug("selection 22, Logging out.")
            break
        else:
            print("Invalid input, please input an integer value corresponding with an option.")
    user.logout()
    exit()  
        
def fetch_movies() -> dict:
    raise NotImplementedError()
    select_from_table("tblMovies")
    
def get_listings():
    currentTime = dt.datetime.now()
    listings = select_from_table_where("tblShowings", f"[{currentTime}] + show_time > sysdatetime()") # returns all data that matches today

def display_listings_cli():
    strListings = pd.DataFrame(get_listings())
    return 

def load_cinema(cinemaID) -> Cinema:
    raise NotImplementedError()

def sanitize(string:str) -> str:
    """
    String sanitization function
    Args:
        string (str): string variable to be sanitized.
    Returns:
        lower case string without any escape chars
    """
    return string.strip().replace('/', '_').replace('\\', '_').lower()

def encrypt(string:str) -> str:
    # add encryption when necessary
    return string

def main_cli() -> None:
    print("\n\nSTARTING CINEMA MANAGEMENT SYSTEM.\n\n")
    # test db is connected....
    if not get_connection():
        logging.critical("Database cannot connect, or dose not exist. \nClosing application.")
        exit()
    # cli loggin attempt
    print("To login please enter your credentials.\n")
    while True:
        username = encrypt(sanitize(input("username: ")))
        password = encrypt(sanitize(input("Password: ")))
        user = loginAttempt(username=username, password=password)
        logging.debug(f"user object={user}")
        
        if user is None:
            print("Login attempt failed please try again.")
        else:
            break

    logging.info(f"Successfuly loged in as {user.username}, loading menu... ")
    logging.debug(f"id={user.user_id}, name={user.username}, pass={user.password_hash}, role={user.role}, cin={user.cinemas}")
    logging.debug(f"{type(user.user_id)}, {type(user.username)}, {type(user.password_hash)}, {type(user.role)}, {type(user.cinemas)}")
    
    
    # based on the different user permision levels, load different menus.
    if user.role=="Manager":
        managerUser_options_cli(user)
    elif user.role=="Admin":
        adminUser_options_cli(user)
    elif user.role=="Staff":
        staffUser_options_cli(user)

# following code will only execute if run directly from this file.
if __name__ == "__main__":
    logging.debug("STARTING main_cli")
    while True:
        main_cli()
    logging.debug("main_cli finished, program exiting. bye bye!")
   
