import logging, json
import datetime as dt
from .constants import DBNAME
from .dbfunc import *
from .cinemaObj import Cinema


# trying to make this a base implementation for all user classes.
class StaffUser:  
    def __init__(self, id:int, name:str, passw:str, role:str, cinemas:list):
        # staff id number assigned by manager during onboarding.
        self.user_id:int = id
        self.username:str = name
        self.password_hash:str = passw
        # staff software level.
        self.role:str = role
        # list of branches active at.
        self.cinemas:list = cinemas
    
    
    def set_attributes(self):
        """ utility function to change user attributes.
        """
        raise NotImplementedError()
    
    # function not working, causes massive recursion?
    #def __repr__(self):
    #"""this function returns this object in a string readable format, eg; print(staffUser) returns 'StaffUser user object(user_id=1, username='user', password_hash='password'... ' """
    #    return f"{self.__class__.__name__} user object(user_id={self.user_id}, username={self.username}, password_hash={self.password_hash}, permissionLevel={self.role})"

    def login(self, username, password_hash) -> None:
        """ Get values for self from db
        """
        logging.debug("called login method")
        data = select_from_tbl_where("tblUsers", f"username='{username}' AND password_hash='{password_hash}'")
        logging.debug(f"loaded data: {data}")
        # this will loop with i as any key from dictData and self.dict that matches, i hope lol.
        for i in data[0].keys(), self.__dict__.keys():
            self.i = dictData[i]
        # replace cinema_ids in self.cinemas with Cinema objects
        for i in self.cinemas:
            self.cinemas[i] = Cinema(id=i)
    
    def _save_self_to_db(self) -> None:
        """ Save all self attributes to db. internal method.
        """
        selfDir = self.__dict__ # pull out class variables and values as dict
        selfDir['user_id'] = selfDir.pop('user_id')
        selfDir['password_hash'] = selfDir.pop('password_hash')
        logging.debug(f"called _save_self_tomethod with data:{selfDir}")
        try:
            update_table(table="tblUsers", condition=f"user_id = '{selfDir['user_id']}'", data=selfDir.pop('user_id'))
        except Exception as e:
            logging.error(e)

    def logout(self) -> None:
        """ Save user to db. external function.
        """
        logging.debug("called logmethod")
        logging.info(f"user {self.username} logging out.")
        #self._save_self_to_db()
    
    def return_film_listings(self):
        """ gets film listings from cinema object
        """
        logging.debug("called returnFilmListimethod")
        listings = []
        for cinema in self.cinemas:
            listings.append(cinema.get_showings())
        return listings
    
    def createTicket(self):
        """ returns ticket version of booking data from db
        """
        raise NotImplementedError()
    
    def createBooking(self, showing_id:int, seats:str):
        """ Adds booking to db and returns ticket if succsessful.
        """
        
        # import showing data
        showing_data = select_from_table_where(table="tblShowings",
                                                selection="*",
                                                condition=f"showing_id={showing_id}")
        showing_data = showing_data[0]
        logging.debug(type(showing_data))
        logging.debug(f"showing_data={showing_data}")
        
        # check if showing_data was returned from db
        if showing_data != None:
            # validate selected seats
            seats = seats.split(" ")
            logging.debug(f"seats.split={seats}")
            seats = [int(x) for x in seats]      # use while seats are int, remove when seats are str.
            logging.debug(f"seats.int={seats}")
            seating_cap = select_from_table_where(table="tblScreens",
                                                    selection="seating_capacity",
                                                    condition=f"screen_id={showing_data['screen_id']}")
            seating_cap = seating_cap[0]['seating_capacity']
            logging.debug(f"seating_cap={seating_cap}")
            
            # check if seat selection in range
            if max(seats) <= int(seating_cap) and min(seats) > 0: # if seats are str this needs reworking.
                # now save to tblBookings
                price = showing_data['price'] * len(seats)
                seats_str = ""
                
                for i in range(len(seats)):
                    seats_str = seats_str + str(seats[i]) + ","
                    
                seats_str = seats_str[:-1]
                insert_data=dict(user_id=self.user_id, showing_id=showing_data['showing_id'], seat_numbers=seats_str, total_price=float(price), booking_date=showing_data['show_time'])
                logging.debug(f"insert_data={insert_data}")
                insert_into_table(table="tblBookings",
                                    data=insert_data)
                return True
                
            return "Invalid seating selection."     # needs to be updated for non cli use

        else:
            return "Invalid showing_id."            # needs to be updated for non cli use
      
    def removeBooking(self, booking_id):
        """ remove bookings from db
        """
        logging.debug("called StaffUser.removeBooking method")
        booking_exists = select_from_table_where(table="tblBookings", selection="booking_id", condition=f"booking_id={booking_id}")
        
        if booking_exists == None:
            return False

        if delete_from_table(table="tblBookings", condition=f"booking_id={booking_id}"):
            return True
        else:
            return "db error."      # needs to be updated for non cli use
        
        


    def veiwBookings(self) -> dict:
        """ Gets bookings from db
        """
        logging.debug("called StaffUser.veiwBookings method")
        data = select_from_table(table="tblBookings") # need to change this to use a mySQL veiw
        logging.debug(data)
        return data
    
    def updateBooking(self, booking_id, **kwargs):
                      #user_id:int=None, showing_id:int=None, seat_numbers:list=None, total_price:float=None, booking_date=None):
        """ updates existing booking from db
        """
        logging.debug("called manageBooking method")
        logging.debug(f"kwargs={kwargs}, {type(kwargs)}")
        set_data = {}
        for key, value in kwargs.items():
            if value:
                set_data.update({key: value})
        
        #dict({key:value} for key, value in kwargs.items() if not value)
        
        
        
        
        """
        if kwargs['user_id'] != None or kwargs['user_id'] != "":
            set_data.update({'user_id':kwargs['user_id']})
        if kwargs['showing_id'] != None or kwargs['showing_id'] != "":
            set_data.update({'showing_id':kwargs['showing_id']})
        if kwargs['seat_numbers'] is not None or kwargs['seat_numbers'] != "":
            set_data.update({'seat_numbers':kwargs['seat_numbers']}) 
        if kwargs['total_price'] is not None or kwargs['total_price'] != "":
            set_data.update({'total_price':kwargs['total_price']})
        if kwargs['booking_date'] is not None or kwargs['booking_date'] != "":
            set_data.update({'booking_date':kwargs['booking_date']})
        """    
        logging.debug(f"set_data={set_data}, booking_id={booking_id}")
        status = update_table("tblBookings", data=set_data, condition=f"booking_id = {booking_id}")
        return status
    
    def veiwShowings(self):
        """ Gets showings from db
        """
        logging.debug("called StaffUser.veiwShowings method")
        data = select_from_table(table="tblShowings") # need to change this to use a mySQL veiw
        logging.debug(data)
        return data
    
    

    

    
    
    
    
# following code will only execute if run directly from this file.
if __name__ == "__main__":
    # testing
    user1 = StaffUser()
    print(user1)